import 'dart:typed_data';

import 'package:flutter/material.dart';
import 'package:flutter_localizations/flutter_localizations.dart';
import 'package:flutter_smart_dialog/flutter_smart_dialog.dart';
import 'package:imsdk_client_core/imsdk_client_core.dart';
import 'package:imsdk_client_flutter/app/widget/widget_loading.dart';

import 'app_config.dart';
import 'app_home.dart';
import 'app_locale.dart';
import 'app_login.dart';
import 'app_webview.dart';
import 'contact_azlist.dart';
import 'contact_editor.dart';
import 'contact_finder.dart';
import 'contact_home.dart';
import 'contact_teaminfo.dart';
import 'contact_userinfo.dart';
import 'session_amap.dart';
import 'session_chat.dart';
import 'session_home.dart';
import 'session_realtime.dart';
import 'session_redpack.dart';
import 'userme_detail.dart';
import 'userme_editor.dart';
import 'userme_home.dart';
import 'userme_settings.dart';
import 'widget/widget_picture.dart';
import 'widget/widget_qrcode.dart';
import 'widget/widget_qrscan.dart';

///
///路由管理与APP入口类
///
class AppRouter extends StatefulWidget {
  /* ******************************** instance content ******************************** */
  ///app名称
  final String title;

  ///已经登录的用户id
  final ObjectId? uid;

  ///已经登录的用户token
  final String? token;

  const AppRouter({Key? key, required this.title, required this.uid, required this.token}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _AppRouterState();

  /* ******************************** public static properties ******************************** */
  ///创建空白页
  static Widget Function() createAppSplashWidget = _createAppSplashWidgetDefault;

  ///创建登录页
  static Widget Function() createAppLoginWidget = _createAppLoginWidgetDefault;

  ///创建APP主页
  static Widget Function() createAppHomeWidget = _createAppHomeWidgetDefault;

  ///创建APP主页->会话列表页
  static Widget Function() createAppHomeSessionHomeWidget = _createAppHomeSessionHomeWidgetDefault;

  ///创建APP主页->自定义页一
  static Widget Function() createAppHomeCustomOneWidget = _createAppHomeCustomOneWidgetDefault;

  ///创建APP主页->自定义页二
  static Widget Function() createAppHomeCustomTwoWidget = _createAppHomeCustomTwoWidgetDefault;

  ///创建APP主页->个人中心页
  static Widget Function() createAppHomeUsermeHomeWidget = _createAppHomeUsermeHomeWidgetDefault;

  ///创建联系人主页->好友列表
  static Widget Function() createContactHomeUserShipsWidget = _createContactHomeUserShipsWidgetDefault;

  ///创建联系人主页->群组列表
  static Widget Function() createContactHomeTeamShipsWidget = _createContactHomeTeamShipsWidgetDefault;

  ///创建联系人主页->申请列表
  static Widget Function() createContactHomeWaitShipsWidget = _createContactHomeWaitShipsWidgetDefault;

  ///应用主页
  static void Function(BuildContext context) pushAppHomePage = _pushAppHomePageDefault;

  ///登录 主页
  static void Function(BuildContext context) pushAppLoginNormalPage = _pushAppLoginNormalPageDefault;

  ///登录 注册页
  static void Function(BuildContext context) pushAppLoginRegisterPage = _pushAppLoginRegisterPageDefault;

  ///登录 修改密码页
  static void Function(BuildContext context) pushAppLoginForgetPwdPage = _pushAppLoginForgetPwdPageDefault;

  ///登录 验证码登录页
  static void Function(BuildContext context) pushAppLoginPhoneCodePage = _pushAppLoginPhoneCodePageDefault;

  ///WebView
  static void Function(BuildContext context, {required AppWebViewType type, String? title, String? url, String? body, bool pushReplacement}) pushAppWebViewPage = _pushAppWebViewPageDefault;

  ///联系人 主页
  static void Function(BuildContext context) pushContactHomePage = _pushContactHomePageDefault;

  ///联系人 搜索页
  static void Function(BuildContext context) pushContactFinderPage = _pushContactFinderPageDefault;

  ///联系人 用户信息页
  static void Function(BuildContext context, {required ObjectId uid, required ObjectId fid, required int from, bool pushReplacement}) pushContactUserInfoPage = _pushContactUserInfoPageDefault;

  ///联系人 群组信息页
  static void Function(BuildContext context, {required ObjectId tid, required ObjectId fid, required int from, bool pushReplacement}) pushContactTeamInfoPage = _pushContactTeamInfoPageDefault;

  ///联系人 信息编辑页
  static void Function(BuildContext context, {required String fieldTitle, required ContactEditorType editorType, required Object targetShip, String? inputLabel, dynamic customType}) pushContactEditorPage = _pushContactEditorPageDefault;

  ///联系人 好友选择页
  static Future<Set<UserShip>?> Function(BuildContext context, {required String title, Team? team}) pushContactUserShipsPickerPage = _pushContactUserShipsPickerPageDefault;

  ///联系人 群组选择页
  static Future<Set<TeamShip>?> Function(BuildContext context, {required String title, Team? team}) pushContactTeamShipsPickerPage = _pushContactTeamShipsPickerPageDefault;

  ///联系人 群组成员页
  static Future<Set<TeamShip>?> Function(BuildContext context, {required String title, required Team team, required bool picker}) pushContactTeamUsersPage = _pushContactTeamUsersPageDefault;

  ///会话 聊天主页
  static void Function(BuildContext context, {required Session session, bool pushAndRemoveUntilFirst}) pushSessionChatPage = _pushSessionChatPageDefault;

  ///会话 实时语音视频页
  static void Function(BuildContext context, {required Message message}) pushSessionRealtimePage = _pushSessionRealtimePageDefault;

  ///会话 高德地图页
  static Future<Location?> Function(BuildContext context, {required SessionAmapType type, Location? location}) pushSessionAmapPage = _pushSessionAmapPageDefault;

  ///会话 发送红包页
  static void Function(BuildContext context, {required ObjectId sid}) pushSessionRedpackPage = _pushSessionRedpackPageDefault;

  ///个人中心 我的信息页
  static void Function(BuildContext context) pushUsermeDetailPage = _pushUsermeDetailPageDefault;

  ///个人中心 会员中心页
  static void Function(BuildContext context) pushUsermeVipCenterPage = _pushUsermeVipCenterPageDefault;

  ///个人中心 我的收藏页
  static void Function(BuildContext context) pushUsermeCollectionPage = _pushUsermeCollectionPageDefault;

  ///个人中心 功能设置页
  static void Function(BuildContext context, {required String title}) pushUsermeSettingsPage = _pushUsermeSettingsPageDefault;

  ///个人中心 信息编辑页
  static void Function(BuildContext context, {required String fieldTitle, required UsermeEditorType editorType, String? inputLabel, dynamic customType}) pushUsermeEditorPage = _pushUsermeEditorPageDefault;

  ///图片 预览页
  static void Function(BuildContext context, {required List<String> urlList, List<String>? heroList, required String Function(String val) onFormatUrl, int? initialPage, ScrollPhysics? scrollPhysics}) pushImageViewerPage = _pushImageViewerPageDefault;

  ///图片 编辑页
  static Future<List<int>?> Function(BuildContext context, {required WidgetPictureType type, required String editTitle, required Uint8List editBytes, required bool cropOnly, required bool outputResize}) pushImageEditorPage = _pushImageEditorPageDefault;

  ///二维码 预览页
  static void Function(BuildContext context, {required String title, required String data, required Icon icon, required String nick, required String hint, Widget? logo}) pushQrcodeViewerPage = _pushQrcodeViewerPageDefalut;

  ///二维码 扫码页
  static void Function(BuildContext context, {required bool Function(BuildContext context, String result) callback}) pushQrcodeScanerPage = _pushQrcodeScanerPageDefault;

  ///协议 用户协议页
  static void Function(BuildContext context) pushProtocolAccountPage = _pushProtocolAccountPageDefault;

  ///协议 隐私政策页
  static void Function(BuildContext context) pushProtocolPrivacyPage = _pushProtocolPrivacyPageDefault;

  ///URL 解析跳转页
  static bool Function(BuildContext context, {required String url, bool pushReplacement}) pushSpecifiedUrlPage = _pushSpecifiedUrlPageDefault;

  /* ******************************** private static functions ******************************** */
  static Widget _createAppSplashWidgetDefault() => const Scaffold(resizeToAvoidBottomInset: false, body: Center(child: FlutterLogo(size: 100)));

  static Widget _createAppLoginWidgetDefault() => const AppLogin(type: AppLoginPageType.normal);

  static Widget _createAppHomeWidgetDefault() => const AppHome();

  static Widget _createAppHomeSessionHomeWidgetDefault() => const SessionHome();

  static Widget _createAppHomeCustomOneWidgetDefault() => const Scaffold(resizeToAvoidBottomInset: false, body: Center(child: Text('Custom page one')));

  static Widget _createAppHomeCustomTwoWidgetDefault() => const Scaffold(resizeToAvoidBottomInset: false, body: Center(child: Text('Custom page two')));

  static Widget _createAppHomeUsermeHomeWidgetDefault() => const UsermeHome();

  static Widget _createContactHomeUserShipsWidgetDefault() => const ContactAZList(key: Key('userships'), type: ContactAZListType.userships);

  static Widget _createContactHomeTeamShipsWidgetDefault() => const ContactAZList(key: Key('teamships'), type: ContactAZListType.teamships);

  static Widget _createContactHomeWaitShipsWidgetDefault() => const ContactAZList(key: Key('waitships'), type: ContactAZListType.waitships);

  static void _pushAppHomePageDefault(BuildContext context) {
    Navigator.pushAndRemoveUntil(context, PageRouteBuilder(pageBuilder: (context, animation1, animation2) => const AppHome(), transitionDuration: Duration.zero), (route) => false);
  }

  static void _pushAppLoginNormalPageDefault(BuildContext context) {
    Navigator.pushAndRemoveUntil(context, PageRouteBuilder(pageBuilder: (context, animation1, animation2) => const AppLogin(type: AppLoginPageType.normal), transitionDuration: Duration.zero), (route) => false);
  }

  static void _pushAppLoginRegisterPageDefault(BuildContext context) {
    Navigator.push(context, MaterialPageRoute(builder: (context) => const AppLogin(type: AppLoginPageType.register)));
  }

  static void _pushAppLoginForgetPwdPageDefault(BuildContext context) {
    Navigator.push(context, MaterialPageRoute(builder: (context) => const AppLogin(type: AppLoginPageType.forgetPwd)));
  }

  static void _pushAppLoginPhoneCodePageDefault(BuildContext context) {
    Navigator.push(context, MaterialPageRoute(builder: (context) => const AppLogin(type: AppLoginPageType.phoneCode)));
  }

  static void _pushAppWebViewPageDefault(BuildContext context, {required AppWebViewType type, String? title, String? url, String? body, bool pushReplacement = false}) {
    if (pushReplacement) {
      Navigator.pushReplacement(context, MaterialPageRoute(builder: (context) => AppWebView(type: type, title: title, url: url, body: body)));
    } else {
      Navigator.push(context, MaterialPageRoute(builder: (context) => AppWebView(type: type, title: title, url: url, body: body)));
    }
  }

  static void _pushContactHomePageDefault(BuildContext context) {
    Navigator.push(context, MaterialPageRoute(builder: (context) => const ContactHome()));
  }

  static void _pushContactFinderPageDefault(BuildContext context) {
    Navigator.push(context, MaterialPageRoute(builder: (context) => const ContactFinder()));
  }

  static void _pushContactUserInfoPageDefault(BuildContext context, {required ObjectId uid, required ObjectId fid, required int from, bool pushReplacement = false}) {
    ContactUserInfo.pushRoute(context, uid: uid, fid: fid, from: from, pushReplacement: pushReplacement);
  }

  static void _pushContactTeamInfoPageDefault(BuildContext context, {required ObjectId tid, required ObjectId fid, required int from, bool pushReplacement = false}) {
    ContactTeamInfo.pushRoute(context, tid: tid, fid: fid, from: from, pushReplacement: pushReplacement);
  }

  static void _pushContactEditorPageDefault(BuildContext context, {required String fieldTitle, required ContactEditorType editorType, required Object targetShip, String? inputLabel, dynamic customType}) {
    Navigator.push(context, MaterialPageRoute(builder: (context) => ContactEditor(fieldTitle: fieldTitle, editorType: editorType, targetShip: targetShip, inputLabel: inputLabel)));
  }

  static Future<Set<UserShip>?> _pushContactUserShipsPickerPageDefault(BuildContext context, {required String title, Team? team}) {
    return Navigator.push<Set<UserShip>>(context, MaterialPageRoute(builder: (context) => ContactAZList(type: ContactAZListType.usershipsPicker, title: title, team: team)));
  }

  static Future<Set<TeamShip>?> _pushContactTeamShipsPickerPageDefault(BuildContext context, {required String title, Team? team}) {
    return Navigator.push<Set<TeamShip>>(context, MaterialPageRoute(builder: (context) => ContactAZList(type: ContactAZListType.teamshipsPicker, title: title, team: team)));
  }

  static Future<Set<TeamShip>?> _pushContactTeamUsersPageDefault(BuildContext context, {required String title, required Team team, required bool picker}) {
    return ContactAZList.pushRouteSingletonForTeamUsers(context, title, team, picker);
  }

  static void _pushSessionChatPageDefault(BuildContext context, {required Session session, bool pushAndRemoveUntilFirst = false}) {
    SessionChat.pushRouteSingleton(context, session: session, pushAndRemoveUntilFirst: pushAndRemoveUntilFirst);
  }

  static void _pushSessionRealtimePageDefault(BuildContext context, {required Message message}) {
    SessionRealTime.pushRouteSingleton(context, message: message);
  }

  static Future<Location?> _pushSessionAmapPageDefault(BuildContext context, {required SessionAmapType type, Location? location}) {
    return Navigator.push<Location?>(context, MaterialPageRoute(builder: (context) => SessionAmap(type: type, location: location)));
  }

  static void _pushSessionRedpackPageDefault(BuildContext context, {required ObjectId sid}) {
    Navigator.push(context, MaterialPageRoute(builder: (context) => SessionRedpack(sid: sid)));
  }

  static void _pushUsermeDetailPageDefault(BuildContext context) {
    Navigator.push(context, MaterialPageRoute(builder: (context) => const UsermeDetail()));
  }

  static void _pushUsermeVipCenterPageDefault(BuildContext context) {
    Navigator.push(context, MaterialPageRoute(builder: (context) => const Scaffold(resizeToAvoidBottomInset: false, body: Center(child: Text('Vip center')))));
  }

  static void _pushUsermeCollectionPageDefault(BuildContext context) {
    Navigator.push(context, MaterialPageRoute(builder: (context) => const Scaffold(resizeToAvoidBottomInset: false, body: Center(child: Text('My Collections')))));
  }

  static void _pushUsermeSettingsPageDefault(BuildContext context, {required String title}) {
    Navigator.push(context, MaterialPageRoute(builder: (context) => UsermeSettings(title: title)));
  }

  static void _pushUsermeEditorPageDefault(BuildContext context, {required String fieldTitle, required UsermeEditorType editorType, String? inputLabel, dynamic customType}) {
    Navigator.push(context, MaterialPageRoute(builder: (context) => UsermeEditor(fieldTitle: fieldTitle, editorType: editorType, inputLabel: inputLabel)));
  }

  static void _pushImageViewerPageDefault(BuildContext context, {required List<String> urlList, List<String>? heroList, required String Function(String val) onFormatUrl, int? initialPage, ScrollPhysics? scrollPhysics}) {
    Navigator.push(
      context,
      PageRouteBuilder(
        pageBuilder: (context, animation1, animation2) => WidgetPicture(type: WidgetPictureType.viewer, urlList: urlList, heroList: heroList, onFormatUrl: onFormatUrl, initialPage: initialPage, scrollPhysics: scrollPhysics),
        transitionsBuilder: (context, animation, secondaryAnimation, Widget child) => FadeTransition(opacity: animation, child: child),
      ),
    );
  }

  static Future<List<int>?> _pushImageEditorPageDefault(BuildContext context, {required WidgetPictureType type, required String editTitle, required Uint8List editBytes, required bool cropOnly, required bool outputResize}) {
    assert(type == WidgetPictureType.editAsJpg || type == WidgetPictureType.editAsPng || type == WidgetPictureType.editAsGif);
    return Navigator.push<List<int>?>(context, MaterialPageRoute(builder: (context) => WidgetPicture(type: type, editTitle: editTitle, editBytes: editBytes, cropOnly: cropOnly, outputResize: outputResize)));
  }

  static void _pushQrcodeViewerPageDefalut(BuildContext context, {required String title, required String data, required Icon icon, required String nick, required String hint, Widget? logo}) {
    Navigator.push(context, MaterialPageRoute(builder: (context) => WidgetQrcode(title: title, data: data, icon: icon, nick: nick, hint: hint, logo: logo)));
  }

  static void _pushQrcodeScanerPageDefault(BuildContext context, {required bool Function(BuildContext context, String result) callback}) {
    Navigator.push(context, MaterialPageRoute(builder: (context) => WidgetQrscan(systemUiOverlayStyle: AppConfig.darkSystemUiOverlayStyle, callback: callback)));
  }

  static void _pushProtocolAccountPageDefault(BuildContext context) {
    Navigator.push(context, MaterialPageRoute(builder: (context) => const Scaffold(resizeToAvoidBottomInset: false, body: Center(child: Text('User agreement')))));
  }

  static void _pushProtocolPrivacyPageDefault(BuildContext context) {
    Navigator.push(context, MaterialPageRoute(builder: (context) => const Scaffold(resizeToAvoidBottomInset: false, body: Center(child: Text('Privacy policy')))));
  }

  static bool _pushSpecifiedUrlPageDefault(BuildContext context, {required String url, bool pushReplacement = false}) {
    final uri = Uri.parse(url);
    AppConfig.logDebug(['uri', uri, uri.scheme, uri.queryParameters, uri.data, uri.host]);
    switch (uri.scheme.toLowerCase()) {
      case 'user':
        AppRouter.pushContactUserInfoPage(context, uid: DbQueryField.hexstr2ObjectId(uri.host), fid: AppConfig.client.user.id, from: Constant.shipFromQrcode, pushReplacement: pushReplacement);
        return true;
      case 'team':
        AppRouter.pushContactTeamInfoPage(context, tid: DbQueryField.hexstr2ObjectId(uri.host), fid: AppConfig.client.user.id, from: Constant.shipFromQrcode, pushReplacement: pushReplacement);
        return true;
      case 'http':
      case 'https':
        AppRouter.pushAppWebViewPage(context, type: AppWebViewType.normal, url: url, pushReplacement: pushReplacement);
        return true;
      default:
        Navigator.pushReplacement(context, MaterialPageRoute(builder: (context) => Scaffold(resizeToAvoidBottomInset: false, appBar: AppBar(), body: Center(child: Text(uri.toString())))));
        return true;
    }
  }
}

class _AppRouterState extends State<AppRouter> with WidgetsBindingObserver {
  late Widget _rootWidget;

  @override
  void initState() {
    super.initState();
    AppConfig.logDebug(['AppRouter.initState()']);
    WidgetsBinding.instance.addObserver(this);
    AppConfig.setOnBrightnessChange(_onBrightnessChangeListener, remove: false);
    if (widget.uid == null || widget.token == null) {
      _rootWidget = AppRouter.createAppLoginWidget();
    } else {
      _rootWidget = AppRouter.createAppSplashWidget();
      _loginByToken();
    }
  }

  @override
  void dispose() {
    AppConfig.logDebug(['AppRouter.dispose()']);
    WidgetsBinding.instance.removeObserver(this);
    AppConfig.setOnBrightnessChange(_onBrightnessChangeListener, remove: true);
    super.dispose();
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    AppConfig.logInfo(['AppRouter.didChangeAppLifecycleState: $state']);
    switch (state) {
      case AppLifecycleState.paused:
        AppConfig.setRunBackground(true);
        break;
      case AppLifecycleState.resumed:
        AppConfig.setRunBackground(false);
        break;
      default:
        break;
    }
  }

  @override
  Widget build(BuildContext context) {
    SmartDialog.config.toast = SmartConfigToast(alignment: Alignment.topCenter, displayType: SmartToastType.last, animationType: SmartAnimationType.scale);
    SmartDialog.config.loading = SmartConfigLoading(clickMaskDismiss: false, backDismiss: false);
    return MaterialApp(
      debugShowCheckedModeBanner: false,
      title: widget.title,
      localizationsDelegates: const [
        AppLocalizations.delegate,
        GlobalMaterialLocalizations.delegate,
        GlobalCupertinoLocalizations.delegate,
        GlobalWidgetsLocalizations.delegate,
      ],
      supportedLocales: const [
        Locale('en'), // English, no country code
        Locale('zh'), // Chinese, no country code
      ],
      theme: AppConfig.themeData,
      home: _rootWidget,
      //全局提示
      navigatorObservers: [FlutterSmartDialog.observer],
      builder: FlutterSmartDialog.init(
        toastBuilder: (String msg) => WidgetLoading(msg: msg, toast: true),
        loadingBuilder: (String msg) => WidgetLoading(msg: msg, toast: false),
      ),
    );
  }

  void _onBrightnessChangeListener() {
    if (mounted) {
      setState(() {});
    }
  }

  void _loginByToken() {
    if (widget.uid == null || widget.token == null) return;
    AppConfig.client.loginByToken(uid: widget.uid!, token: widget.token!).then((value) {
      if (mounted) {
        setState(() {
          _rootWidget = value.ok ? AppRouter.createAppHomeWidget() : AppRouter.createAppLoginWidget();
        });
      }
    });
  }
}
