import 'package:flutter/material.dart';
import 'package:imsdk_client_core/imsdk_client_core.dart';

import 'app_config.dart';
import 'app_locale.dart';
import 'app_router.dart';
import 'widget/widget_avator.dart';
import 'widget/widget_badge.dart';
import 'widget/widget_divider.dart';

///
///个人中心主页
///
class UsermeHome extends StatefulWidget {
  const UsermeHome({Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _UsermeHomeState();
}

class _UsermeHomeState extends State<UsermeHome> {
  static const _heroUsermeHomeHeadTag = '_heroUsermeHomeHeadTag';
  @override
  void initState() {
    super.initState();
    AppConfig.logDebug(['UsermeHome.initState()']);
    AppConfig.client.setOnUserUpdateListener(_onUserUpdateListener, remove: false);
  }

  @override
  void dispose() {
    AppConfig.logDebug(['UsermeHome.dispose()']);
    AppConfig.client.setOnUserUpdateListener(_onUserUpdateListener, remove: true);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final themeData = Theme.of(context);
    return Scaffold(
      appBar: _buildAppBar(themeData),
      body: Column(
        children: [
          Expanded(child: _buildBody(themeData)),
          WidgetDivider.forFull,
        ],
      ),
    );
  }

  AppBar _buildAppBar(ThemeData themeData) {
    return AppBar(
      backgroundColor: themeData.cardColor,
      leading: IconButton(
        icon: Icon(themeData.brightness == Brightness.light ? Icons.brightness_4_outlined : Icons.brightness_5_outlined),
        tooltip: AppLocalizations.usermeHomeActionBtnAppThemeTip,
        onPressed: () => AppConfig.setBrightness(themeData.brightness == Brightness.light),
      ),
      title: Text(AppLocalizations.appHomeTabItemUserme),
      actions: [
        IconButton(
          icon: const Icon(Icons.qr_code_outlined, size: AppConfig.bigGlobalIconReSize),
          tooltip: AppLocalizations.usermeHomeActionBtnMyQrCodeTip,
          onPressed: () => AppRouter.pushQrcodeViewerPage(
            context,
            title: AppLocalizations.usermeHomeActionBtnMyQrCodeTip,
            data: AppConfig.formatIdentUrl(user: AppConfig.client.user),
            icon: AppConfig.getUserSexIcon(AppConfig.client.user.sex),
            nick: ComTools.formatUserNick(AppConfig.client.user),
            hint: AppLocalizations.usermeHomeActionBtnMyQrcodeHint,
            logo: WidgetAvator(type: WidgetAvatorType.large, url: AppConfig.client.user.icon, urlList: AppConfig.client.user.head, onFormatUrl: AppConfig.formatAttachUrl),
          ),
        ),
      ],
    );
  }

  Widget _buildBody(ThemeData themeData) {
    return ListView(
      physics: AppConfig.bodyGlobalScrollPhysics,
      children: [
        Ink(
          color: themeData.cardColor,
          child: Column(
            children: [
              ListTile(
                contentPadding: AppConfig.infoListTileHeadPadding,
                leading: InkWell(
                  child: Hero(tag: _heroUsermeHomeHeadTag, child: WidgetAvator(type: WidgetAvatorType.large, url: AppConfig.client.user.icon, urlList: AppConfig.client.user.head, onFormatUrl: AppConfig.formatAttachUrl)),
                  onTap: () => AppRouter.pushImageViewerPage(context, urlList: [AppConfig.client.user.icon], heroList: [_heroUsermeHomeHeadTag], onFormatUrl: AppConfig.formatAttachUrl),
                ),
                title: Text(ComTools.formatUserNick(AppConfig.client.user), style: themeData.textTheme.subtitle1?.copyWith(fontWeight: FontWeight.bold), overflow: TextOverflow.ellipsis),
                subtitle: Text(
                  '${AppLocalizations.usermeHomeListHeadMyNoTitle}${ComTools.formatUserNo(AppConfig.client.user, empty: AppLocalizations.usermeHomeListHeadMyNoEmpty)}',
                  style: themeData.textTheme.subtitle1,
                  overflow: TextOverflow.ellipsis,
                ),
                trailing: WidgetBadge(count: AppConfig.countUserNotSettings(), dotOnly: true, align: WidgetBadgeAlign.centerLeft, margin: WidgetBadge.hugeMargin, icon: AppConfig.infoListTileTailArrowIC),
                onTap: () => AppRouter.pushUsermeDetailPage(context),
              ),
              WidgetDivider.forFull,
            ],
          ),
        ),
        AppConfig.cardGlobalSpaceMedium,
        Ink(
          color: themeData.cardColor,
          child: Column(
            children: [
              WidgetDivider.forFull,
              ListTile(
                contentPadding: AppConfig.infoListTileItemPadding,
                horizontalTitleGap: 0,
                leading: const Icon(Icons.home_outlined, color: Colors.green),
                title: Text(AppLocalizations.usermeHomeListItemMyVipCentTitle),
                trailing: AppConfig.infoListTileTailArrowIC,
                onTap: () => AppRouter.pushUsermeVipCenterPage(context),
              ),
              WidgetDivider.forListTileLeadingIcon,
              ListTile(
                contentPadding: AppConfig.infoListTileItemPadding,
                horizontalTitleGap: 0,
                leading: const Icon(Icons.grade_outlined, color: Colors.orange),
                title: Text(AppLocalizations.usermeHomeListItemMyCollectTitle),
                trailing: AppConfig.infoListTileTailArrowIC,
                onTap: () => AppRouter.pushUsermeCollectionPage(context),
              ),
              WidgetDivider.forListTileLeadingIcon,
              ListTile(
                contentPadding: AppConfig.infoListTileItemPadding,
                horizontalTitleGap: 0,
                leading: const Icon(Icons.info_outlined, color: Colors.cyan),
                title: Text(AppLocalizations.usermeHomeListItemProtoUserTitle),
                trailing: AppConfig.infoListTileTailArrowIC,
                onTap: () => AppRouter.pushProtocolAccountPage(context),
              ),
              WidgetDivider.forListTileLeadingIcon,
              ListTile(
                contentPadding: AppConfig.infoListTileItemPadding,
                horizontalTitleGap: 0,
                leading: const Icon(Icons.gpp_maybe_outlined, color: Colors.purpleAccent),
                title: Text(AppLocalizations.usermeHomeListItemProtoPrivTitle),
                trailing: AppConfig.infoListTileTailArrowIC,
                onTap: () => AppRouter.pushProtocolPrivacyPage(context),
              ),
              WidgetDivider.forFull,
            ],
          ),
        ),
        AppConfig.cardGlobalSpaceMedium,
        Ink(
          color: themeData.cardColor,
          child: Column(
            children: [
              WidgetDivider.forFull,
              ListTile(
                contentPadding: AppConfig.infoListTileItemPadding,
                horizontalTitleGap: 0,
                leading: const Icon(Icons.settings_outlined, color: Colors.blue),
                title: Text(AppLocalizations.usermeHomeListItemSettingsTitle),
                trailing: AppConfig.infoListTileTailArrowIC,
                onTap: () => AppRouter.pushUsermeSettingsPage(context, title: AppLocalizations.usermeHomeListItemSettingsTitle),
              ),
              WidgetDivider.forFull,
            ],
          ),
        ),
      ],
    );
  }

  void _onUserUpdateListener(EasyPacket packet) {
    if (mounted) {
      setState(() {});
    }
  }
}
